package net.wanji.utc.hisense.controller;

/**
 * @author Kent HAN
 * @date 2022/12/1 15:07
 */

import com.fasterxml.jackson.databind.JsonMappingException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.hisense.pojo.convert.HisenseLightStatusPojo;
import net.wanji.utc.hisense.service.SignalStatusService;
import org.dom4j.DocumentException;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * 运行状态、告警、灯态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "运行状态、告警、灯态信息接口", tags = "运行状态、告警、灯态信息接口")
@RequestMapping("/signalStatus")
@RestController
@RequiredArgsConstructor
@Slf4j
public class SignalStatusController {
    private final SignalStatusService signalStatusService;

    @AspectLog(description = "运行状态、告警数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/runningStatusAlarm", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "运行状态、告警数据", notes = "运行状态、告警数据", response = SignalStatusLogPO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SignalStatusLogPO.class)
    })
    public JsonViewObject runningStatusAlarm(String crossId) {
        List<SignalStatusLogPO> signalStatusLogPOList = null;
        try {
            signalStatusLogPOList = signalStatusService.runningStatusAlarm(crossId);
        } catch (DocumentException e) {
            throw new RuntimeException(e);
        }
        return JsonViewObject.newInstance().success(signalStatusLogPOList);

    }

    @AspectLog(description = "灯态数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatus", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据", notes = "灯态数据", response = LightsStatusVO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO.class)
    })
    public JsonViewObject lightStatus(String crossId) {
        List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus(crossId);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        return jsonViewObject.success(lightsStatusVOList);
    }

    @AspectLog(description = "接收海信灯态", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/receiveLightStatus", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "接收海信灯态", notes = "接收海信灯态", response = LightsStatusVO.class, produces = MediaType.APPLICATION_JSON)
    public JsonViewObject receiveLightStatus(@RequestBody List<HisenseLightStatusPojo> hisenseLightStatusPojos) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            signalStatusService.receiveLightStatus(hisenseLightStatusPojos);
        } catch (JsonMappingException e) {
        }
        return jsonViewObject.success("海信灯态接收成功");
    }
}
