package net.wanji.utc.hisense.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.CrossSchemeStageOptLogPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossSchemeStageOptLogMapper;
import net.wanji.databus.dto.CrossSchemeRingsDTO;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.vo.*;
import net.wanji.utc.hisense.common.OperationBaseDom;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.pojo.dto.RingPhaseDTO;
import net.wanji.utc.hisense.pojo.dto.StagePhaseDTO;
import net.wanji.databus.po.CrossSchemeRings;
import net.wanji.utc.hisense.service.ControlCommandService;
import net.wanji.utc.hisense.service.StaticInfoService;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.apache.commons.lang3.ObjectUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2023/05/08 10:43
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class ControlCommandServiceImpl implements ControlCommandService {
    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private StaticInfoService staticInfoService;
    @Resource
    private StaticInfoServiceImpl staticInfoServiceImpl;
    @Resource
    private CrossSchemeStageOptLogMapper crossSchemeStageOptLogMapper;

    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        // 海信方案下发
        // 下发xml
        return null;
    }

    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(commandVO.getCrossCode());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        if (commandVO.getCommand() == 1) {
            //锁相
            messageContent.addElement(HttpConstants.COMMAND).setText("1");
        } else if (commandVO.getCommand() == 0) {
            //解锁
            messageContent.addElement(HttpConstants.COMMAND).setText("0");
        }
        //相位驻留
        String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        log.info("第一次锁相/解锁 海信返回msg:{}",xmlPost);
        /*String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";*/
        if (StringUtils.isNotBlank(xmlPost)) {
            Document extendPost = DocumentHelper.parseText(xmlPost);
            String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
            if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                if (commandVO.getCommand() == 1){
                    Integer duration = commandVO.getDuration();
                    Thread.sleep(duration * 1000);
                    messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                    boolean isOk = sendMesIsOk(document.asXML());
                    if (isOk) {
                        return jsonViewObject.success("相位锁定/解锁 - 控制成功" );
                    } else {
                        return jsonViewObject.fail("相位锁定/解锁 - 控制失败 - 相位锁定成功，但解锁失败 ");
                    }
                }
                return jsonViewObject.success("相位锁定/解锁 - 控制成功");
            } else {
                return jsonViewObject.fail("相位锁定/解锁 - 控制失败 - 第一次给海信下发指令失败");
            }
        }else {
            return jsonViewObject.fail("相位锁定/解锁 - 控制失败 - 第一次给海信下发指令失败");
        }
//        return jsonViewObject.fail("相位锁定/解锁 - 控制失败");
    }

    @Override
    public JsonViewObject stepControl(String code, Integer command, Integer stepNum) throws Exception {
        if (ObjectUtils.isEmpty(stepNum) || stepNum == 0) {
            //默认步进1次
            stepNum = 1;
        }
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(code);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.COMMAND).setText(command + "");
        //海信现在只支持0
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (command == 0) {
            //取消步进
            long start = System.currentTimeMillis();
            boolean isOk = sendMesIsOk(document.asXML());
            long end = System.currentTimeMillis();
            log.info("取消步进 耗时: {} ms", end - start);
//            boolean isOk = true;
            if (isOk) {
                return jsonViewObject.success("取消步进 - 控制成功");
            } else {
                return jsonViewObject.fail("取消步进 - 控制失败");
            }
        } else if (command == 1) {
            //逻辑：第一次发送步进指令是相位驻留，以后发送才是相位步进，步进成功则取消步进。步进失败，也立即下发取消步进控制
            //相位驻留
            long start = System.currentTimeMillis();
            String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
            long end = System.currentTimeMillis();
            log.info("相位驻留 耗时: {} ms,msg:{}", end - start,xmlPost);
            /*String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";*/

            if (StringUtils.isNotBlank(xmlPost)) {
                Document extendPost = DocumentHelper.parseText(xmlPost);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    //开始步进
                    //给海信发送http请求
                    for (int i = 0; i < stepNum; i++) {
                        long s = System.currentTimeMillis();
                        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                        long e = System.currentTimeMillis();
                        log.info("相位步进 耗时: {} ms,msg:{}", e - s,post);
                        /*String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                                "    <subSystem>Hisense</subSystem>\n" +
                                "    <messageType>12</messageType>\n" +
                                "    <isRequest>0</isRequest>\n" +
                                "    <seq>20241103155205549244</seq>\n" +
                                "    <needResponse>1</needResponse>\n" +
                                "    <result>1</result>\n" +
                                "    <flag>0</flag>\n" +
                                "    <tStamp>1731037114609</tStamp>\n" +
                                "    <messageContent>\n" +
                                "        <Spot>311358</Spot>\n" +
                                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                                "    </messageContent>\n" +
                                "</systemScription>";*/
                        if (StringUtils.isNotBlank(post)) {
                            Document postResult = DocumentHelper.parseText(post);
                            String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                            if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                                log.info("路口号 ：{}，第 {} 次步进成功,result:{}", code, i + 1, result);
                            } else {
                                log.error("路口号 ：{}，第 {} 次步进失败,result:{}", code, i + 1, result);
                                messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                                //取消步进
                                long s1 = System.currentTimeMillis();
                                boolean isOk = sendMesIsOk(document.asXML());

                                long e1 = System.currentTimeMillis();
                                log.info("取消 相位步进 耗时: {} ms", e1 - s1);

//                                boolean isOk = true;
                                if (isOk) {
                                    return jsonViewObject.fail("第 "+ (i+1) +" 次相位步进失败，取消步进成功，路口号： " + code);
                                } else {
                                    return jsonViewObject.fail("第 "+ (i+1) +" 次相位步进失败，取消步进失败，路口号： " + code);
                                }
                            }
                            //停顿10ms
                            Thread.sleep(10);
                        } else {
                            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                            //取消步进
                            long s1 = System.currentTimeMillis();
                            boolean isOk = sendMesIsOk(document.asXML());
                            long e1 = System.currentTimeMillis();
                            log.info("取消 相位步进 耗时: {} ms,isOK:{}", e1 - s1,isOk);

//                            boolean isOk = true;
                            if (isOk) {
                                return jsonViewObject.fail("第 "+(i+1)+" 次相位步进失败，但取消步进成功，路口号： " + code);
                            } else {
                                return jsonViewObject.fail("第 "+(i+1)+" 次相位步进失败，取消步进失败，路口号： " + code);
                            }
                        }

                    }
                    messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                    //取消步进
                    long s1 = System.currentTimeMillis();
                    boolean isOk = sendMesIsOk(document.asXML());
                    long e1 = System.currentTimeMillis();
                    log.info("取消 相位步进 耗时: {} ms,isOK:{}", e1 - s1,isOk);

//                    boolean isOk = true;
                    if (isOk) {
                        return jsonViewObject.success("相位步进成功，路口号： " + code);
                    } else {
                        return jsonViewObject.fail("相位步进成功，但取消步进失败，路口号： " + code);
                    }
                }else {
                    return jsonViewObject.fail("相位步进 - 控制失败 - 相位驻留失败");
                }
            } else {
                return jsonViewObject.fail("相位步进 - 控制失败 - 相位驻留失败");
            }
        }
        return jsonViewObject.fail("相位步进 - 控制失败 - 第一次请求海信接口报错");

    }

    public JsonViewObject stepControlBak(String code, Integer command, Integer stepNum) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(code);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.COMMAND).setText(command + "");
        //目前海信步进幅度只支持0
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        if (stepNum == 0) {
            //顺序步进，循环一次
            stepNum = 1;
        }
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (command == 0) {
            //取消步进
//            boolean isOk = sendMesIsOk(document.asXML());
            boolean isOk = true;
            if (isOk) {
                return jsonViewObject.success("取消步进 - 控制成功");
            } else {
                return jsonViewObject.fail("取消步进 - 控制失败");
            }
        } else {
            //相位驻留
//            String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
            String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";
            if (StringUtils.isNotBlank(xmlPost)) {
                Document extendPost = DocumentHelper.parseText(xmlPost);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    //开始步进
                    //给海信发送http请求
                    for (int i = 0; i < stepNum; i++) {
//                        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                        String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                                "    <subSystem>Hisense</subSystem>\n" +
                                "    <messageType>12</messageType>\n" +
                                "    <isRequest>0</isRequest>\n" +
                                "    <seq>20241103155205549244</seq>\n" +
                                "    <needResponse>1</needResponse>\n" +
                                "    <result>1</result>\n" +
                                "    <flag>0</flag>\n" +
                                "    <tStamp>1731037114609</tStamp>\n" +
                                "    <messageContent>\n" +
                                "        <Spot>311358</Spot>\n" +
                                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                                "    </messageContent>\n" +
                                "</systemScription>";
                        if (StringUtils.isNotBlank(post)) {
                            Document postResult = DocumentHelper.parseText(post);
                            String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                            if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                                log.info("路口号 ：{}，第 {} 次步进成功,result:{}", code, i + 1, result);
                            } else {
                                log.error("路口号 ：{}，第 {} 次步进失败,result:{}", code, i + 1, result);
                                messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                                //取消步进
//                                boolean isOk = sendMesIsOk(document.asXML());
                                boolean isOk = true;
                                if (isOk) {
                                    return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + code);
                                } else {
                                    return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + code);
                                }
                            }
                            //停顿10ms
                            Thread.sleep(10);
                        } else {
                            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                            //取消步进
//                            boolean isOk = sendMesIsOk(document.asXML());
                            boolean isOk = true;
                            if (isOk) {
                                return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + code);
                            } else {
                                return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + code);
                            }
                        }
                    }
                }
            } else {
                return jsonViewObject.fail("相位步进 - 控制失败");
            }
        }
        return jsonViewObject.success("相位步进 - 控制成功");
    }

    @Override
    public JsonViewObject setSignalControl(ControlCommandVO commandVO) throws Exception {
        return null;
    }

    @Override
    public JsonViewObject recoverSchedule(String code) throws Exception {
        log.info("接收到恢复固定配时，code:{}", code);
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(code);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_16);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText("0");

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        /*String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";*/
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(post)) {
            log.error(" 手动方案控制- 恢复固定配时 失败");
            return jsonViewObject.fail(" 手动方案控制- 恢复固定配时 失败");
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();

        if (HttpConstants.RESULT_1.equals(result)) {
            //指令下发成功
            jsonViewObject.success("恢复固定配置 - 指令下发成功");
        } else if (HttpConstants.RESULT_0.equals(result)) {
            //指令下发失败
            jsonViewObject.fail("恢复固定配置 - 指令下发失败");
        } else if (HttpConstants.RESULT_2.equals(result)) {
            //指令执行成功
            jsonViewObject.success("恢复固定配置 - 指令执行成功");
        } else {
            //指令执行失败
            jsonViewObject.fail("恢复固定配置 - 指令执行失败");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        //TODO 海信相位配时下发
        return null;
    }

    @Override
    public JsonViewObject phaseDiffSend() {
        return null;
    }

    @Override
    public JsonViewObject tempSchemeDetail(TempSchemeSendVO tempSchemeSendVO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(tempSchemeSendVO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_39);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText(tempSchemeSendVO.getPattern());
        messageContent.addElement(HttpConstants.CYCLE).setText(tempSchemeSendVO.getCycle());
        messageContent.addElement(HttpConstants.OFFSET).setText(tempSchemeSendVO.getOffset());
        messageContent.addElement(HttpConstants.TYPE).setText(tempSchemeSendVO.getType());
        //卫博传的是空格
        messageContent.addElement(HttpConstants.SPLIT).setText(tempSchemeSendVO.getSplit().replaceAll(" ", ","));

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        /*String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";*/
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(post)) {
            log.error("HTTP请求 5.16. 方案优化控制 失败");
            return jsonViewObject.fail("HTTP请求 5.16. 方案优化控制 失败");
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();

        if (HttpConstants.RESULT_1.equals(result)) {
            //指令下发成功
            jsonViewObject.success("临时方案 - 指令下发成功");
            log.info("临时方案下发成功,tempSchemeSendVO:{}", JSONObject.toJSONString(tempSchemeSendVO));
        } else if (HttpConstants.RESULT_0.equals(result)) {
            log.error("临时方案下发失败,tempSchemeSendVO:{}", JSONObject.toJSONString(tempSchemeSendVO));
            //指令下发失败
            jsonViewObject.fail("临时方案 - 指令下发失败");
        } else if (HttpConstants.RESULT_2.equals(result)) {
            log.info("临时方案指令执行成功,tempSchemeSendVO:{}", JSONObject.toJSONString(tempSchemeSendVO));
            //指令执行成功
            jsonViewObject.success("临时方案 - 指令执行成功");
        } else {
            log.error("临时方案指令执行失败,tempSchemeSendVO:{}", JSONObject.toJSONString(tempSchemeSendVO));
            //指令执行失败
            jsonViewObject.fail("临时方案 - 指令执行失败");
        }
        Integer code = jsonViewObject.getCode();
        int optResult = 1;
        if (code != 200) {
            optResult = 2;
        }
        //方案优化记录
        crossSchemeOptLogBak(tempSchemeSendVO, tempSchemeSendVO.getSplit(), optResult);
        return jsonViewObject;
    }

    /**
     * @return void
     * @Description 方案优化记录
     * @Param tempSchemeSendVO  环图查询参数
     * @Param split              优化的绿信比
     * @Param result            下发结果
     **/
    private void crossSchemeOptLog(TempSchemeSendVO tempSchemeSendVO, String greenTime, int result) throws Exception {
        CrossSchemeRingsDTO crossSchemeRingsDTO = new CrossSchemeRingsDTO();
        String crossId = tempSchemeSendVO.getCrossId();
        String pattern = tempSchemeSendVO.getPattern();
        crossSchemeRingsDTO.setCrossId(crossId);
        crossSchemeRingsDTO.setPattern(pattern);
        CrossSchemeStageOptLogPO crossSchemeStageOptLogPO = new CrossSchemeStageOptLogPO();
        crossSchemeStageOptLogPO.setCrossId(crossId);
        long currentTimeMillis = System.currentTimeMillis();
        //方案号处理
        int schemeId = (Integer.valueOf(crossSchemeRingsDTO.getPattern()) + 2) / 3;
        crossSchemeStageOptLogPO.setSchemeId(schemeId);
        crossSchemeStageOptLogPO.setOptResult(result);
        crossSchemeStageOptLogPO.setDataInfo(JSONObject.toJSONString(tempSchemeSendVO));
        crossSchemeStageOptLogPO.setBatchTime(currentTimeMillis);
        crossSchemeStageOptLogMapper.insert(crossSchemeStageOptLogPO);
    }

    /**
     * @return void
     * @Description 方案优化记录
     * @Param tempSchemeSendVO  环图查询参数
     * @Param split              优化的绿信比
     * @Param result            下发结果
     **/
    private void crossSchemeOptLogBak(TempSchemeSendVO tempSchemeSendVO, String greenTime, int result) throws Exception {
        CrossSchemeRingsDTO crossSchemeRingsDTO = new CrossSchemeRingsDTO();
        String crossId = tempSchemeSendVO.getCrossId();
        String pattern = tempSchemeSendVO.getPattern();
        crossSchemeRingsDTO.setCrossId(crossId);
        crossSchemeRingsDTO.setPattern(pattern);
        boolean isOk = true;

        //方案环图请求
        CrossSchemeRings crossSchemeRings = staticInfoService.crossSchemeRings(crossSchemeRingsDTO);
        long currentTimeMillis = System.currentTimeMillis();
        if (ObjectUtil.isNotEmpty(crossSchemeRings) && result == 1) {
            //环数
            int ringCount = Integer.parseInt(crossSchemeRings.getRingCount());
            JSONObject cycleJSONObject = crossSchemeRings.getCycleList();
            List<CrossSchemeRings.Phase> phaseList = crossSchemeRings.getPhaseList();
            CrossSchemeRings.Phase phase = phaseList.get(0);
            //红灯黄灯时间给默认值0
            phase.setRedTime(getDefaultPhaseTime());
            phase.setYellowTime(getDefaultPhaseTime());
            //绿灯时间为神思绿信比时间
            phase.setGreenTime(greenTime);
            List<StagePhaseDTO> stagePhaseDTOList = null;
            try {
                //封装环和环上相位数据
                Map<Integer, List<RingPhaseDTO>> ringPhasesMap = staticInfoServiceImpl.getRingPhasesMap(ringCount, cycleJSONObject, phase);
                //环封装阶段
                stagePhaseDTOList = staticInfoServiceImpl.stagePhaseList(Integer.valueOf(tempSchemeSendVO.getCycle()), ringCount, ringPhasesMap);
            } catch (Exception e) {
                CrossSchemeStageOptLogPO crossSchemeStageOptLogPO = new CrossSchemeStageOptLogPO();
                crossSchemeStageOptLogPO.setCrossId(crossId);
                //方案号处理
                int schemeId = (Integer.valueOf(crossSchemeRingsDTO.getPattern()) + 2) / 3;
                crossSchemeStageOptLogPO.setSchemeId(schemeId);
                crossSchemeStageOptLogPO.setOptResult(2);
                crossSchemeStageOptLogPO.setDataInfo(JSONObject.toJSONString(tempSchemeSendVO));
                crossSchemeStageOptLogPO.setBatchTime(currentTimeMillis);
                crossSchemeStageOptLogMapper.insert(crossSchemeStageOptLogPO);
                log.error("绿信比有误!");
                throw new RuntimeException(e);
            }
            if (ObjectUtil.isNotEmpty(stagePhaseDTOList)) {
                for (StagePhaseDTO stagePhaseDTO : stagePhaseDTOList) {
                    CrossSchemeStageOptLogPO crossSchemeStageOptLogPO = new CrossSchemeStageOptLogPO();
                    crossSchemeStageOptLogPO.setCrossId(crossId);
                    //方案号处理
                    int schemeId = (Integer.valueOf(crossSchemeRingsDTO.getPattern()) + 2) / 3;
                    crossSchemeStageOptLogPO.setSchemeId(schemeId);
                    crossSchemeStageOptLogPO.setPhaseNo(stagePhaseDTO.getStageId() + "");
                    crossSchemeStageOptLogPO.setPhaseTime(stagePhaseDTO.getStageTime());
                    crossSchemeStageOptLogPO.setOptResult(1);
                    crossSchemeStageOptLogPO.setDataInfo(JSONObject.toJSONString(tempSchemeSendVO));
                    crossSchemeStageOptLogPO.setBatchTime(currentTimeMillis);
                    List<RingPhaseDTO> ringPhaseList = stagePhaseDTO.getRingPhaseList();
                    List<Integer> list = new ArrayList<>();
                    for (RingPhaseDTO ringPhaseDTO : ringPhaseList) {
                        if (ObjectUtil.isNotEmpty(ringPhaseDTO)) {
                            list.add(ringPhaseDTO.getPhaseId());
                        }
                    }
                    if (ObjectUtil.isNotEmpty(list)) {
                        String phases = CollectionUtil.join(list, ",");
                        crossSchemeStageOptLogPO.setPhaseList(phases);
                    }
                    crossSchemeStageOptLogMapper.insert(crossSchemeStageOptLogPO);
                }
            } else {
                isOk = false;
            }
        } else {
            isOk = false;

        }
        if (!isOk) {
            CrossSchemeStageOptLogPO crossSchemeStageOptLogPO = new CrossSchemeStageOptLogPO();
            crossSchemeStageOptLogPO.setCrossId(crossId);
            //方案号处理
            int schemeId = (Integer.valueOf(crossSchemeRingsDTO.getPattern()) + 2) / 3;
            crossSchemeStageOptLogPO.setSchemeId(schemeId);
            crossSchemeStageOptLogPO.setOptResult(2);
            crossSchemeStageOptLogPO.setDataInfo(JSONObject.toJSONString(tempSchemeSendVO));
            crossSchemeStageOptLogPO.setBatchTime(currentTimeMillis);
            crossSchemeStageOptLogMapper.insert(crossSchemeStageOptLogPO);
        }
    }

    /**
     * @return java.lang.String
     * @Description 默认的相位时间
     * @Param []
     **/
    private String getDefaultPhaseTime() {
        StringBuilder str = new StringBuilder();
        for (int i = 0; i < 16; i++) {
            str.append("0").append(" ");
        }
        StringBuilder stringBuilder = str.deleteCharAt(str.lastIndexOf("") - 1);
        return stringBuilder.toString();
    }

    @Override
    public JsonViewObject extendPhase(ExtendPhaseDTO extendPhaseDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(extendPhaseDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        //相位驻留
        messageContent.addElement(HttpConstants.COMMAND).setText("1");
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        /*String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                "    <subSystem>Hisense</subSystem>\n" +
                "    <messageType>12</messageType>\n" +
                "    <isRequest>0</isRequest>\n" +
                "    <seq>20241103155205549244</seq>\n" +
                "    <needResponse>1</needResponse>\n" +
                "    <result>1</result>\n" +
                "    <flag>0</flag>\n" +
                "    <tStamp>1731037114609</tStamp>\n" +
                "    <messageContent>\n" +
                "        <Spot>311358</Spot>\n" +
                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                "    </messageContent>\n" +
                "</systemScription>";*/
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(post)) {
            log.error("HTTP请求 5.12. 步进及取消控制 失败");
            return jsonViewObject.fail("HTTP请求 5.12. 步进及取消控制 失败");
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();

        if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
            //执行成功，开始延长
            Thread.sleep(extendPhaseDTO.getExtendTime() * 1000);
            //延长结束，解除步进
            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
            //给海信发送http请求
            String post1 = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
            /*String post1 = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";*/
            if (StringUtils.isNotBlank(post1)) {
                Document extendPost = DocumentHelper.parseText(post1);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    return jsonViewObject.success("延长相位 - 控制成功");
                }
            }
            //延长失败,解除步进控制,发起三次取消控制请求
            log.error("HTTP请求 5.12. 步进及取消控制 失败");
            //重新发起三次取消控制请求
            for (int j = 0; j < 3; j++) {
                String httpPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                /*String httpPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                        "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                        "    <subSystem>Hisense</subSystem>\n" +
                        "    <messageType>12</messageType>\n" +
                        "    <isRequest>0</isRequest>\n" +
                        "    <seq>20241103155205549244</seq>\n" +
                        "    <needResponse>1</needResponse>\n" +
                        "    <result>1</result>\n" +
                        "    <flag>0</flag>\n" +
                        "    <tStamp>1731037114609</tStamp>\n" +
                        "    <messageContent>\n" +
                        "        <Spot>311358</Spot>\n" +
                        "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                        "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                        "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                        "    </messageContent>\n" +
                        "</systemScription>";*/
                if (StringUtils.isNotBlank(httpPost)) {
                    Document extendPost = DocumentHelper.parseText(httpPost);
                    String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                    if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                        return jsonViewObject.success("延长相位 - 控制成功");
                    }
                }
                Thread.sleep(1000);
            }

        } else {
            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
            //重新发起三次取消控制请求
            for (int j = 0; j < 3; j++) {
                String httpPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                if (StringUtils.isNotBlank(httpPost)) {
                    Document extendPost = DocumentHelper.parseText(httpPost);
                    String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                    if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                        return jsonViewObject.success("延长相位 - 控制成功");
                    }
                }
                Thread.sleep(1000);
            }
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepingPhase(StepingPhaseDTO stepingPhaseDTO) throws Exception {
        if (ObjectUtils.isEmpty(stepingPhaseDTO.getStepCount()) || stepingPhaseDTO.getStepCount() == 0) {
            //默认步进1次
            stepingPhaseDTO.setStepCount(1);
        }
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(stepingPhaseDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_22);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.COMMAND).setText(stepingPhaseDTO.getCommand() + "");
        //海信现在只支持0
        messageContent.addElement(HttpConstants.CTRLSTEP).setText("0");
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (stepingPhaseDTO.getCommand() == 0) {
            //取消步进
            boolean isOk = sendMesIsOk(document.asXML());
//            boolean isOk = true;
            if (isOk) {
                return jsonViewObject.success("取消步进 - 控制成功");
            } else {
                return jsonViewObject.fail("取消步进 - 控制失败");
            }
        } else if (stepingPhaseDTO.getCommand() == 1) {
            //逻辑：第一次发送步进指令是相位驻留，以后发送才是相位步进，步进成功则取消步进。步进失败，也立即下发取消步进控制
            //相位驻留
            String xmlPost = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
           /* String xmlPost = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                    "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                    "    <subSystem>Hisense</subSystem>\n" +
                    "    <messageType>12</messageType>\n" +
                    "    <isRequest>0</isRequest>\n" +
                    "    <seq>20241103155205549244</seq>\n" +
                    "    <needResponse>1</needResponse>\n" +
                    "    <result>1</result>\n" +
                    "    <flag>0</flag>\n" +
                    "    <tStamp>1731037114609</tStamp>\n" +
                    "    <messageContent>\n" +
                    "        <Spot>311358</Spot>\n" +
                    "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                    "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                    "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                    "    </messageContent>\n" +
                    "</systemScription>";*/

            if (StringUtils.isNotBlank(xmlPost)) {
                Document extendPost = DocumentHelper.parseText(xmlPost);
                String extendResult = extendPost.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(extendResult) || HttpConstants.RESULT_2.equals(extendResult)) {
                    //开始步进
                    //给海信发送http请求
                    for (int i = 0; i < stepingPhaseDTO.getStepCount(); i++) {
                        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
                        /*String post = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                "<systemScription  System=\"TCIP\" Version=\"1.0\">\n" +
                                "    <subSystem>Hisense</subSystem>\n" +
                                "    <messageType>12</messageType>\n" +
                                "    <isRequest>0</isRequest>\n" +
                                "    <seq>20241103155205549244</seq>\n" +
                                "    <needResponse>1</needResponse>\n" +
                                "    <result>1</result>\n" +
                                "    <flag>0</flag>\n" +
                                "    <tStamp>1731037114609</tStamp>\n" +
                                "    <messageContent>\n" +
                                "        <Spot>311358</Spot>\n" +
                                "        <CoordPatternStatus>4</CoordPatternStatus>\n" +
                                "        <CoordCycleStatus>160</CoordCycleStatus>\n" +
                                "        <CoordSyncStatus>123</CoordSyncStatus>\n" +
                                "    </messageContent>\n" +
                                "</systemScription>";*/
                        if (StringUtils.isNotBlank(post)) {
                            Document postResult = DocumentHelper.parseText(post);
                            String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                            if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                                log.info("路口号 ：{}，第 {} 次步进成功,result:{}", stepingPhaseDTO.getCrossId(), i + 1, result);
                            } else {
                                log.error("路口号 ：{}，第 {} 次步进失败,result:{}", stepingPhaseDTO.getCrossId(), i + 1, result);
                                messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                                //取消步进
                                boolean isOk = sendMesIsOk(document.asXML());
//                                boolean isOk = true;
                                if (isOk) {
                                    return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + stepingPhaseDTO.getCrossId());
                                } else {
                                    return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + stepingPhaseDTO.getCrossId());
                                }
                            }
                            //停顿10ms
                            Thread.sleep(10);
                        } else {
                            messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                            //取消步进
                            boolean isOk = sendMesIsOk(document.asXML());
//                            boolean isOk = true;
                            if (isOk) {
                                return jsonViewObject.fail("相位步进失败，取消步进成功，路口号： " + stepingPhaseDTO.getCrossId());
                            } else {
                                return jsonViewObject.fail("相位步进失败，取消步进失败，路口号： " + stepingPhaseDTO.getCrossId());
                            }
                        }

                    }
                    messageContent.selectSingleNode(HttpConstants.COMMAND).setText("0");
                    //取消步进
                    boolean isOk = sendMesIsOk(document.asXML());
//                    boolean isOk = true;
                    if (isOk) {
                        return jsonViewObject.success("相位步进成功，，路口号： " + stepingPhaseDTO.getCrossId());
                    } else {
                        return jsonViewObject.fail("相位步进成功，但取消步进失败，路口号： " + stepingPhaseDTO.getCrossId());
                    }
                }
            } else {
                return jsonViewObject.fail("相位步进 - 控制失败");
            }
        }
        return jsonViewObject.fail("相位步进 - 控制失败");
    }

    private boolean sendMesIsOk(String xml) throws DocumentException, InterruptedException {
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, xml);
        if (StringUtils.isBlank(post)) {
            return false;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
            return true;
        } else {
            //连续重试发三次
            for (int i = 0; i < 3; i++) {
                post = OkHttpClientUtil.xmlPost(hisenseUrl, xml);
                postResult = DocumentHelper.parseText(post);
                result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
                if (HttpConstants.RESULT_1.equals(result) || HttpConstants.RESULT_2.equals(result)) {
                    return true;
                }
                Thread.sleep(1000);
            }
        }
        return false;
    }
}
