package net.wanji.utc.hisense.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.ScheduleUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.BaseCrossSchemeMapper;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossSchemeStageOptLogMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.dto.*;
import net.wanji.databus.po.CoordinationStatus;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.CrossInfoPOExt;
import net.wanji.databus.po.CrossNowSchemePO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import net.wanji.utc.hisense.cache.CrossInfoCache;
import net.wanji.utc.hisense.common.*;
import net.wanji.utc.hisense.common.constants.HttpConstants;
import net.wanji.utc.hisense.mapper.*;
import net.wanji.utc.hisense.pojo.dto.RingPhaseDTO;
import net.wanji.utc.hisense.pojo.dto.StagePhaseDTO;
import net.wanji.databus.po.CrossSchemeRings;
import net.wanji.utc.hisense.pojo.view.*;
import net.wanji.utc.hisense.service.StaticInfoService;
import net.wanji.utc.hisense.util.CommonUtils;
import net.wanji.utc.hisense.util.OkHttpClientUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/05/08 17:29
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class StaticInfoServiceImpl implements StaticInfoService {

    @Value("${hisense.url}")
    private String hisenseUrl;
    @Resource
    private CrossInfoMapper crossInfoMapper;
    @Resource
    private ManufacturerInfoMapper manufacturerInfoMapper;
    @Resource
    private VIntersectionMapper vIntersectionMapper;
    @Resource
    private VNtcipTimeBaseScheduleMapper vNtcipTimeBaseScheduleMapper;
    @Resource
    private VNtcipTimeBaseDayplanMapper vNtcipTimeBaseDayplanMapper;
    @Resource
    private VNtcipTimeBaseAscActionMapper vNtcipTimeBaseAscActionMapper;
    @Resource
    private VNtcipChannelViewMapper vNtcipChannelViewMapper;
    @Resource
    private VNtcipPatternMapper vNtcipPatternMapper;
    @Resource
    private CrossSchemeStageOptLogMapper crossSchemeStageOptLogMapper;
    @Resource
    private BaseCrossSchemeMapper baseCrossSchemeMapper;
    public static  Map<String, Integer> planMap = new HashMap<>();

    @Override
    public List<CrossInfoPO> crossBasicInfo(CrossInfoDTO crossInfoDTO) throws Exception {
        Integer id = manufacturerInfoMapper.selectIdByCode(crossInfoDTO.getManufacturerCode());
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(id);
        List<String> codeList = crossInfoPOList.stream().map(CrossInfoPO::getCode).collect(Collectors.toList());
        Map<String, String> codeCrossIdMap = crossInfoPOList.stream().collect(Collectors.toMap(CrossInfoPO::getCode, CrossInfoPO::getId));
        List<CrossInfoPO> crossInfoPOS = new ArrayList<>();
        if (StringUtils.equalsIgnoreCase("HISENSE", crossInfoDTO.getManufacturerCode())) {
            LambdaQueryWrapper<VIntersectionView> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(VIntersectionView::getCIntsID, codeList);
            List<VIntersectionView> vIntersectionEntities = vIntersectionMapper.selectList(queryWrapper);
            for (VIntersectionView vIntersectionView : vIntersectionEntities) {
                String cIntsID = vIntersectionView.getCIntsID();
                CrossInfoPO crossInfoPO = new CrossInfoPO();
                crossInfoPO.setId(codeCrossIdMap.get(cIntsID));
                crossInfoPO.setCode(cIntsID);
                crossInfoPO.setName(vIntersectionView.getCIntsName());
                crossInfoPO.setIp(vIntersectionView.getCCONTROLLERIP());
                String[] array = {String.valueOf(vIntersectionView.getLongitude()), String.valueOf(vIntersectionView.getLatitude())};
                crossInfoPO.setLocation(StringUtils.join(array, ","));
                crossInfoPO.setVersion(vIntersectionView.getCControllerModel());
                crossInfoPO.setManufacturerId(22);
                crossInfoPOS.add(crossInfoPO);
            }
        }
        return crossInfoPOS;
    }

    @Override
    public SchemePhaseLightsVO schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        String crossId = schemePhaseLightsDTO.getCrossId();
        SchemePhaseLightsVO phaseLightsVO = new SchemePhaseLightsVO();
        List<CrossLightsPO> crossLightsList = new ArrayList<>();
        List<CrossPhasePO> crossPhaseList = new ArrayList<>();
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = new ArrayList<>();
        List<CrossSchemePO> crossSchemeList = new ArrayList<>();
        //查询该路口的所有方案
        //从缓存中拿路口信息
        CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoCache().get(crossId);
        LambdaQueryWrapper<VNtcipPatternView> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(VNtcipPatternView::getCIntsID, crossInfoPO.getCode());
        List<VNtcipPatternView> vNtcipPatternViews = vNtcipPatternMapper.selectList(queryWrapper);
        AtomicInteger lightNum = new AtomicInteger(0);
        //灯组缓存
        Map<String, Integer> lightsMap = new HashMap<>();
        for (VNtcipPatternView vNtcipPatternView : vNtcipPatternViews) {
            CrossSchemeRingsDTO crossSchemeRingsDTO = new CrossSchemeRingsDTO();
            crossSchemeRingsDTO.setCrossId(crossId);
            crossSchemeRingsDTO.setPattern(vNtcipPatternView.getNPatternNumber() + "");
            //查询方案环图请求
            CrossSchemeRings crossSchemeRings = crossSchemeRingsHisen(crossSchemeRingsDTO);
            if (Optional.ofNullable(crossSchemeRings).isPresent()) {
                //环数
                int ringCount = Integer.parseInt(crossSchemeRings.getRingCount());
                //周期长度
                String cycleLen = crossSchemeRings.getCycleLen();
                JSONObject cycleJSONObject = crossSchemeRings.getCycleList();

                //相位差
                String offset = crossSchemeRings.getOffset();
                int cycle = Integer.valueOf(cycleLen);
                CrossSchemePO crossSchemePO = new CrossSchemePO();
                //方案号
                int schemeNo = (Integer.valueOf(crossSchemeRings.getPattern()) + 2) / 3;
                crossSchemePO.setSchemeNo(schemeNo + "");
                crossSchemePO.setName("方案" + schemeNo);
                crossSchemePO.setCrossId(crossId);
                crossSchemePO.setCycle(cycle);
                //协调相位id先默认给1
                crossSchemePO.setCoordPhase(1);
                crossSchemePO.setOffset(Integer.valueOf(offset));
                //来源默认信号机
                crossSchemePO.setSource(1);
                //默认给0
                crossSchemePO.setIsDeleted(0);
                //默认给0
                crossSchemePO.setStatus(0);
                crossSchemePO.setGmtCreate(null);
                crossSchemePO.setGmtModified(null);
                crossSchemeList.add(crossSchemePO);

                List<CrossSchemeRings.Phase> phaseList = crossSchemeRings.getPhaseList();
                CrossSchemeRings.Phase phase = phaseList.get(0);

                //封装环和环上相位数据
                Map<Integer, List<RingPhaseDTO>> ringPhasesMap = getRingPhasesMap(ringCount, cycleJSONObject, phase);
                //16 个相位的放行方向，每个相位由 8 字节整型值存储（最多支持 7 个放行方向），每个字节表示一个放行方向，
                String[] channelDim = phase.getChannelDim().split(" ");
                //数据处理
                getStagesByRings(lightsMap, lightNum, cycle, ringCount, crossId, schemeNo, ringPhasesMap, crossLightsList, crossPhaseList, crossPhaseLightsPOList, channelDim);
            }
        }
        phaseLightsVO.setCrossPhaseLightsPOList(crossPhaseLightsPOList);
        phaseLightsVO.setCrossLightsList(crossLightsList);
        CrossSchemePO crossSchemePO = addNewScheme(crossId);
        crossSchemeList.add(crossSchemePO);
        phaseLightsVO.setCrossSchemeList(crossSchemeList);
        phaseLightsVO.setCrossPhaseList(crossPhaseList);
        return phaseLightsVO;
    }
    //初始化一个黃闪方案
    private CrossSchemePO addNewScheme(String crossId){
        CrossSchemePO crossSchemePO = new CrossSchemePO();
        //方案号
        int schemeNo = 85;
        crossSchemePO.setSchemeNo(schemeNo + "");
        crossSchemePO.setName("方案" + schemeNo);
        crossSchemePO.setCrossId(crossId);
        crossSchemePO.setCycle(0);
        //协调相位id先默认给1
        crossSchemePO.setCoordPhase(1);
        crossSchemePO.setOffset(0);
        //来源默认信号机
        crossSchemePO.setSource(1);
        //默认给0
        crossSchemePO.setIsDeleted(0);
        //默认给0
        crossSchemePO.setStatus(0);
        crossSchemePO.setGmtCreate(null);
        crossSchemePO.setGmtModified(null);
        return crossSchemePO;
    }

    /**
     * @return java.util.Map<java.lang.Integer, java.util.List < net.wanji.utc.hisense.pojo.dto.RingPhaseDTO>>
     * @Description 封装环和环上相位数据
     * @Param ringCount         环的数量
     * @Param cycleJSONObject    环数据
     * @Param redTime    红
     * @Param greenTime  绿
     * @Param yellowTime 黄
     **/
    public Map<Integer, List<RingPhaseDTO>> getRingPhasesMap(int ringCount, JSONObject cycleJSONObject, CrossSchemeRings.Phase phaseDto) {
        //16 个相位的红灯时间（单位秒）,中间用空格隔开,不存在的相位置 0。
        String[] redTime = phaseDto.getRedTime().split(" ");
        //16 个相位的绿灯时间（单位秒）,中间用空格隔开,不存在的相位置 0。
        String[] greenTime = phaseDto.getGreenTime().split(" ");
        //16 个相位的黄灯时间（单位秒）,中间用空格隔开,不存在的相位置 0。
        String[] yellowTime = phaseDto.getYellowTime().split(" ");
        //16 个相位的最小绿时间（单位秒）,中间用空格隔开,不存在的相位置 0。
        String[] minGTime = phaseDto.getMinGTime().split(" ");
        //16 个相位的最大绿时间（单位秒）,中间用空格隔开,不存在的相位置 0。
        String[] maxGTime = phaseDto.getMaxGTime().split(" ");

        Map<Integer, List<RingPhaseDTO>> map = new HashMap<>();
        for (int i = 1; i <= ringCount; i++) {
            //取出环中的数据，并去掉空格和_，得到相位
            String ringPhases = cycleJSONObject.getString("Cycle" + i);

            List<Integer> phaseIdList = new ArrayList<>();
            String[] split = ringPhases.split(" ");
            for (int i1 = 0; i1 < split.length; i1++) {
                String phase = split[i1];
                String[] split1 = phase.split("_");
                for (String s : split1) {
                    phaseIdList.add(Integer.parseInt(s));

                }
            }
            List<RingPhaseDTO> ringPhaseDTOS = new ArrayList<>();
            for (Integer phaseId : phaseIdList) {
                RingPhaseDTO ringPhase = new RingPhaseDTO();
                int red = Integer.valueOf(redTime[phaseId - 1]);
                int green = Integer.valueOf(greenTime[phaseId - 1]);
                int yellow = Integer.valueOf(yellowTime[phaseId - 1]);
                int minG = Integer.valueOf(minGTime[phaseId - 1]);
                int maxG = Integer.valueOf(maxGTime[phaseId - 1]);
                ringPhase.setPhaseId(phaseId);
                ringPhase.setPhaseStatus(0);
                ringPhase.setRedTime(red);
                ringPhase.setGreenTime(green);
                ringPhase.setYellowTime(yellow);
                ringPhase.setMaxGTime(maxG);
                ringPhase.setMinGTime(minG);
                ringPhase.setPhaseTime(red + green + yellow);
                ringPhaseDTOS.add(ringPhase);
            }
            map.put(i, ringPhaseDTOS);
        }
        return map;
    }

    /**
     * @return null
     * @Description 数据处理
     * @Param ringPhaseMap   环包含的相位
     * @Param phasesTime     各个相位的红绿黄时间和
     **/
    private void getStagesByRings(Map<String, Integer> lightsMap, AtomicInteger lightNum, int cycle, int ringCount, String crossId, int schemeNo, Map<Integer, List<RingPhaseDTO>> ringPhasesMap, List<CrossLightsPO> crossLightsList, List<CrossPhasePO> crossPhaseList, List<CrossPhaseLightsPO> crossPhaseLightsPOList, String[] channelDim) {
        //环转阶段
        List<StagePhaseDTO> stagePhaseList = stagePhaseList(cycle, ringCount, ringPhasesMap);
        if (CollectionUtil.isNotEmpty(stagePhaseList)) {
            //将结果组装到结果集
            doView(lightsMap, lightNum, cycle, ringCount, crossId, schemeNo, stagePhaseList, crossLightsList, crossPhaseList, crossPhaseLightsPOList, channelDim);
        }
    }

    /**
     * @return java.util.List<net.wanji.utc.hisense.pojo.dto.StagePhaseDTO>
     * @Description 环封装阶段
     * @Param [cycle, ringCount, ringPhasesMap]
     **/
    public List<StagePhaseDTO> stagePhaseList(int cycle, int ringCount, Map<Integer, List<RingPhaseDTO>> ringPhasesMap) {
        //已经计算的阶段时长
        int stageTime = 0;
        //循环次数,也代表阶段号
        int count = 1;
        //阶段对应的相位
        List<StagePhaseDTO> stagePhaseList = new ArrayList<>();
        while (stageTime < cycle) {
            int firstPhaseTime = 0;
            RingPhaseDTO firstRingPhaseDTO = null;
            //key为环号，value为该数据在List<RingPhaseDTO>的序号，为了更新List<RingPhaseDTO>的status
            Map<Integer, Integer> minStageMap = new HashMap<>();
            List<RingPhaseDTO> stageList = new ArrayList<>();
            StagePhaseDTO stagePhase = new StagePhaseDTO();
            //第一个阶段
            if (count == 1) {
                for (int i = 1; i <= ringCount; i++) {
                    //第i个环上的数据
                    List<RingPhaseDTO> ringPhaseDTOS = ringPhasesMap.get(i);
                    //第一个相位的数据
                    RingPhaseDTO ringPhaseDTO = ringPhaseDTOS.get(0);
                    Integer phaseTime = ringPhaseDTO.getPhaseTime();
                    if (firstPhaseTime == 0) {
                        firstPhaseTime = phaseTime;
                        firstRingPhaseDTO = ringPhaseDTO;
                        minStageMap.put(i, 0);
                    } else if (phaseTime <= firstPhaseTime) {
                        if (phaseTime < firstPhaseTime) {
                            //清空缓存
                            minStageMap.clear();
                        }
                        firstPhaseTime = phaseTime;
                        firstRingPhaseDTO = ringPhaseDTO;
                        minStageMap.put(i, 0);
                    }
                    //每个环的第一个相位一定在第一个阶段
                    stageList.add(ringPhaseDTO);
                }
            } else {
                for (int i = 1; i <= ringCount; i++) {
                    //第i个环上的数据
                    List<RingPhaseDTO> ringPhaseDTOS = ringPhasesMap.get(i);
                    //如果该环上相位都没有执行，则取第0个相位，否则取状态最后一次出现的1的下一个相位
                    List<RingPhaseDTO> collect = ringPhaseDTOS.stream().filter(x -> x.getPhaseStatus() == 1).collect(Collectors.toList());
                    if (CollectionUtil.isNotEmpty(collect)) {
                        //取最后一位
                        RingPhaseDTO phaseDTO = collect.get(collect.size() - 1);
                        //状态最后一次出现的1的下一个相位
                        int index = ringPhaseDTOS.lastIndexOf(phaseDTO) + 1;
                        RingPhaseDTO ringPhaseDTO = null;
                        if (index >= ringPhaseDTOS.size()) {
                            firstRingPhaseDTO = ringPhaseDTOS.get(ringPhaseDTOS.size() - 1);
                            int phaseTime = cycle - stageTime;
                            firstPhaseTime = phaseTime;
                            minStageMap.put(i, index - 1);
                        } else {
                            ringPhaseDTO = ringPhaseDTOS.get(index);
                            //计算该相位及走过的所有时间
                            int totalTime = ringPhaseDTOS.stream().filter(x -> ringPhaseDTOS.indexOf(x) <= index).mapToInt(RingPhaseDTO::getPhaseTime).sum();
                            //totalTime - stageTime 为该相位剩余时间
                            Integer phaseTime = totalTime - stageTime;
                            if (firstPhaseTime == 0) {
                                firstPhaseTime = phaseTime;
                                firstRingPhaseDTO = ringPhaseDTO;
                                minStageMap.put(i, index);
                            } else {
                                if (phaseTime <= firstPhaseTime) {
                                    if (phaseTime < firstPhaseTime) {
                                        //清空缓存
                                        minStageMap.clear();
                                    }
                                    firstPhaseTime = phaseTime;
                                    firstRingPhaseDTO = ringPhaseDTO;
                                    minStageMap.put(i, index);
                                }
                            }
                        }

                        stageList.add(ringPhaseDTO);
                    } else {
                        //在第一个相位
                        RingPhaseDTO ringPhaseDTO = ringPhaseDTOS.get(0);
                        //该相位一定会走
                        stageList.add(ringPhaseDTO);
                        //第一个相位的时间减去走过的时长，则为剩余时长，和其他时间做比较
                        Integer phaseTime = ringPhaseDTO.getPhaseTime() - stageTime;
                        if (firstPhaseTime == 0) {
                            //第一次进入循环
                            firstPhaseTime = phaseTime;
                            firstRingPhaseDTO = ringPhaseDTO;
                            minStageMap.put(i, 0);
                        } else if (phaseTime <= firstPhaseTime) {
                            if (phaseTime < firstPhaseTime) {
                                if (!minStageMap.isEmpty()) {
                                    //清空缓存
                                    minStageMap.clear();
                                }
                            }
                            firstPhaseTime = phaseTime;
                            firstRingPhaseDTO = ringPhaseDTO;
                            minStageMap.put(i, 0);
                        }
                    }
                }
            }

            stageTime += firstPhaseTime;
            Set<Integer> keySet = minStageMap.keySet();
            for (Integer integer : keySet) {
                Integer no = minStageMap.get(integer);
                //更新ringPhasesMap状态
                ringPhasesMap.get(integer).get(no).setPhaseStatus(1);
            }
            stagePhase.setStageId(count);
            stagePhase.setRingPhaseList(stageList);
            stagePhase.setStageTime(firstPhaseTime);
            stagePhaseList.add(stagePhase);
            count++;
        }
        return stagePhaseList;
    }

    /**
     * @return void
     * @Description 组装结果集
     * @Param [cycle, ringCount, crossId, schemeNo, crossLightsList, crossPhaseList, crossPhaseLightsPOList, channelDim]
     **/
    private void doView(Map<String, Integer> lightsMap, AtomicInteger lightNum, int cycle, int ringCount, String crossId, int schemeNo, List<StagePhaseDTO> stagePhaseList, List<CrossLightsPO> crossLightsList, List<CrossPhasePO> crossPhaseList, List<CrossPhaseLightsPO> crossPhaseLightsPOList, String[] channelDim) {
        AtomicInteger phaseNum = new AtomicInteger(0);
        //遍历阶段
        for (StagePhaseDTO stagePhaseDTO : stagePhaseList) {
            Integer stageId = stagePhaseDTO.getStageId();
            Integer stageTime = stagePhaseDTO.getStageTime();
            //遍历相位
            List<RingPhaseDTO> ringPhaseList = stagePhaseDTO.getRingPhaseList();
            //相位
            CrossPhasePO crossPhasePO = new CrossPhasePO();
            //相位号变为方案号拼接阶段号
            stageId = Integer.valueOf(String.valueOf(schemeNo).concat(String.valueOf(stageId)));
            crossPhasePO.setPhaseNo(String.valueOf(stageId));
            crossPhasePO.setName("相位" + stageId);
            phaseNum.getAndIncrement();
            crossPhasePO.setSort(phaseNum.get());
            crossPhasePO.setCrossId(crossId);
            crossPhasePO.setPlanId(schemeNo);
            crossPhasePO.setAccompanyPhaseNo("1");
            crossPhasePO.setRingNo(1);
            crossPhasePO.setGroupNo(1);
            crossPhasePO.setPhaseType(1);
            if (schemeNo == 85) {
                //黃闪
                crossPhasePO.setControlMode(7);
            } else {
                crossPhasePO.setControlMode(1);
            }
            crossPhasePO.setPhaseTime(stageTime);
            Integer index = null;
            List<RingPhaseDTO> collect = ringPhaseList.stream().filter(x -> x.getYellowTime() + x.getRedTime() + x.getGreenTime() == stageTime).collect(Collectors.toList());
            //首先取红绿黄灯和的时长=阶段时长，再次取有红灯时长的相位
            if (CollectionUtil.isNotEmpty(collect)) {
                List<RingPhaseDTO> collect1 = collect.stream().filter(x -> x.getRedTime() != 0).collect(Collectors.toList());
                RingPhaseDTO ringPhaseDTO = null;
                if (CollectionUtil.isNotEmpty(collect1)) {
                    ringPhaseDTO = collect1.get(0);
                } else {
                    ringPhaseDTO = collect.get(0);
                }
                index = ringPhaseList.indexOf(ringPhaseDTO);
            }
            if (index != null) {
                crossPhasePO.setGreenTime(ringPhaseList.get(index).getGreenTime());
                crossPhasePO.setYellowTime(ringPhaseList.get(index).getYellowTime());
                crossPhasePO.setRedTime(ringPhaseList.get(index).getRedTime());
                crossPhasePO.setMinGreenTime(ringPhaseList.get(index).getMinGTime());
                crossPhasePO.setMaxGreenTime(ringPhaseList.get(index).getMaxGTime());
            } else {
                //如果没有匹配到，则取有红灯相位的红灯，黄灯时间，通过阶段时长算出绿灯时长
                List<RingPhaseDTO> collect1 = ringPhaseList.stream().filter(x -> x.getRedTime() != 0).collect(Collectors.toList());
                RingPhaseDTO phaseDTO = null;
                if (CollectionUtil.isNotEmpty(collect1)) {
                    phaseDTO = collect1.get(0);
                } else {
                    phaseDTO = ringPhaseList.get(0);
                }
                Integer redTime = phaseDTO.getRedTime();
                Integer yellowTime = phaseDTO.getYellowTime();
                crossPhasePO.setYellowTime(yellowTime);
                crossPhasePO.setRedTime(redTime);
                crossPhasePO.setMinGreenTime(phaseDTO.getMinGTime());
                crossPhasePO.setMaxGreenTime(phaseDTO.getMaxGTime());
                crossPhasePO.setGreenTime(stageTime - redTime - yellowTime);
            }

            crossPhasePO.setGreenFlashTime(0);
            crossPhasePO.setYellowFlashTime(0);
            crossPhasePO.setRedFlashTime(0);
            crossPhasePO.setPedFlashTime(0);

            crossPhasePO.setGmtCreate(null);
            crossPhasePO.setGmtModified(null);
            crossPhaseList.add(crossPhasePO);

            for (RingPhaseDTO phaseDTO : ringPhaseList) {
                Integer phaseId = phaseDTO.getPhaseId();
                long channe = Long.valueOf(channelDim[phaseId - 1]);
                String binaryString = Long.toBinaryString(channe);
                //获取方向转向分组
                List<String> binaryList = CommonUtils.groupStringFromRight(binaryString);
                for (String readUtf8Line : binaryList) {
                    // 高三位 方向
                    String dir = readUtf8Line.substring(0, 3);
                    // 低五位 转向
                    String turn = readUtf8Line.substring(3);
                    Integer wjLightsDir = HiseseDirEnum.getWjLightsDir(Integer.parseInt(dir, 2));
                    Integer wjLightsTurn = HisenseTurnEnum.getWjLightsTurn(Integer.parseInt(turn, 2));
                    if (wjLightsDir == 0 && wjLightsTurn == 0){
                        continue;
                    }
                    Integer wjPerson = HisensePersonEntranceAndExitEnum.getWjperson(Integer.parseInt(turn, 2));
                    //灯组
                    CrossLightsPO crossLightsPO = new CrossLightsPO();

                    //默认0
                    crossLightsPO.setType(0);
                    if (wjPerson != 0) {
                        crossLightsPO.setType(3);
                    } else if (wjPerson == 0 && wjLightsDir > 0) {
                        crossLightsPO.setType(1);
                    } else if (wjLightsDir == 7) {
                        crossLightsPO.setType(1);
                    }

                    //key
                    String lightMapKey = String.valueOf(wjLightsDir).concat(String.valueOf(wjLightsTurn)).concat(String.valueOf(crossLightsPO.getType()));
                    if (!lightsMap.containsKey(lightMapKey)) {
                        lightNum.getAndIncrement();
                        lightsMap.put(lightMapKey, lightNum.get());
                        crossLightsPO.setLightsNo(String.valueOf(lightsMap.get(lightMapKey)));
                        crossLightsPO.setName("灯组" + lightsMap.get(lightMapKey));
                        crossLightsPO.setDir(wjLightsDir);
                        crossLightsPO.setTurn(wjLightsTurn);
                        crossLightsPO.setCrossId(crossId);
                        crossLightsPO.setInOutType(wjPerson);
                        crossLightsPO.setGmtCreate(null);
                        crossLightsPO.setGmtModified(null);
                        crossLightsList.add(crossLightsPO);
                    }
                    //灯组与相位关系
                    CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                    crossPhaseLightsPO.setLightsId(lightsMap.get(lightMapKey));
                    crossPhaseLightsPO.setPhaseId(stageId); // 方向转向
                    crossPhaseLightsPO.setCrossId(crossId);
                    crossPhaseLightsPO.setGmtCreate(null);
                    crossPhaseLightsPO.setGmtModified(null);
                    crossPhaseLightsPOList.add(crossPhaseLightsPO);
                }
            }
        }
    }

    @NotNull
    private static CrossPhaseLightsPO getCrossPhaseLightsPO(String crossId, VNtcipChannelPhaseDTO vNtcipChannelPhaseDTO, Integer nChannelNumber) {
        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
        crossPhaseLightsPO.setPhaseId(vNtcipChannelPhaseDTO.getNPhaseNumber());
        CrossLightsPO selectEntity = new CrossLightsPO();
        selectEntity.setCrossId(crossId);
        selectEntity.setLightsNo(String.valueOf(nChannelNumber));
        crossPhaseLightsPO.setLightsId(null);
        crossPhaseLightsPO.setCrossId(crossId);
        return crossPhaseLightsPO;
    }

    private static CrossLightsPO getCrossLights(String crossId, Integer nChannelControlType, Integer nChannelNumber, Integer nChannelDim) {
        CrossLightsPO crossLightsPO = new CrossLightsPO();
        crossLightsPO.setLightsNo(String.valueOf(nChannelNumber));
        crossLightsPO.setName("海信灯组" + nChannelNumber);
        if (Objects.equals(ChannelControlEnum.PEDESTRIAN.getCode(), nChannelControlType)) {
            crossLightsPO.setType(3);
        }
        if (Objects.equals(ChannelControlEnum.VEHICLE.getCode(), nChannelControlType)) {
            crossLightsPO.setType(1);
        }
        String dirTurn = String.format("%8s", Integer.toBinaryString(nChannelDim)).replace(' ', '0');
        if (StringUtils.isNotBlank(dirTurn)) {
            String dir = dirTurn.substring(0, 3); // 高三位 方向
            String turn = dirTurn.substring(3); // 低五位 转向
            crossLightsPO.setDir(HiseseDirEnum.getWjLightsDir(Integer.parseInt(dir, 2)));
            crossLightsPO.setTurn(HisenseTurnEnum.getWjLightsTurn(Integer.parseInt(turn, 2)));
            crossLightsPO.setCrossId(crossId);
        }
        return crossLightsPO;
    }

    @Override
    public List<PlanSectionVO> planSection(PlanSectionDTO planSectionDTO) throws Exception {
        String crossId = planSectionDTO.getCrossId();
        //从缓存中拿路口信息
        CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoCache().get(crossId);
        if (Optional.ofNullable(crossInfoPO).isPresent()) {
            String code = crossInfoPO.getCode();
            List<VNtcipTimeBaseDayplanDTO> vNtcipTimeBaseDayplanDTOS = vNtcipTimeBaseDayplanMapper.listByCrossIdAndDayPlanId(code, planSectionDTO.getPlanNo());
            if (!CollectionUtils.isEmpty(vNtcipTimeBaseDayplanDTOS)) {
                List<PlanSectionVO> list = new ArrayList<>();
                //根据时段表号分组
                Map<Integer, List<VNtcipTimeBaseDayplanDTO>> collect = vNtcipTimeBaseDayplanDTOS.stream().collect(Collectors.groupingBy(VNtcipTimeBaseDayplanDTO::getNDayPlanNumber));
                List<CrossPlanPO> crossPlanPOList = new ArrayList<>();

                Set<Integer> keySet = collect.keySet();
                for (Integer i : keySet) {
                    CrossPlanPO crossPlan = new CrossPlanPO();
                    crossPlan.setCrossId(crossId);
                    //计划号-取的是时段表号
                    crossPlan.setPlanNo(String.valueOf(i));
                    crossPlan.setName("计划号:" + i);
                    crossPlanPOList.add(crossPlan);

                    PlanSectionVO planSectionVO = new PlanSectionVO();
                    List<CrossSectionPO> crossSectionPOList = new ArrayList<>();
                    //时段数据
                    List<VNtcipTimeBaseDayplanDTO> dayplanDTOS = collect.get(i).stream().sorted(Comparator.comparing(VNtcipTimeBaseDayplanDTO::getNDayPlanEventNumber)).collect(Collectors.toList());
                    for (int i1 = 0; i1 < dayplanDTOS.size(); i1++) {
                        VNtcipTimeBaseDayplanDTO dayplanDTO = dayplanDTOS.get(i1);
                        CrossSectionPO crossSectionPO = new CrossSectionPO();
                        crossSectionPO.setCrossId(crossId);
                        //时段号 - 取的是时段序号
                        crossSectionPO.setSectionNo(dayplanDTO.getNDayPlanEventNumber() + "");
                        crossSectionPO.setStartTime(String.format("%02d", Integer.valueOf(dayplanDTO.getNDayPlanHour())) + ":" + String.format("%02d", Integer.valueOf(dayplanDTO.getNDayPlanMinute())));
                        if (i1 == dayplanDTOS.size() - 1) {
                            crossSectionPO.setEndTime("23:59");
                        } else {
                            VNtcipTimeBaseDayplanDTO dto = dayplanDTOS.get(i1 + 1);
                            crossSectionPO.setEndTime(String.format("%02d", Integer.valueOf(dto.getNDayPlanHour())) + ":" + String.format("%02d", Integer.valueOf(dto.getNDayPlanMinute())));
                        }
                        // 计划ID - 取的是时段表号
                        crossSectionPO.setPlanId(dayplanDTO.getNDayPlanNumber());
                        //方案号
                        Integer pattern = dayplanDTO.getNTimebaseAscPattern();
                        if (pattern == 85) {
                            //黃闪
                            crossSectionPO.setControlMode(3);
                        } else {
                            //定周期
                            crossSectionPO.setControlMode(1);
                        }
                        //方案ID
                        crossSectionPO.setSchemeId(pattern);
                        //创建时间
                        crossSectionPO.setGmtCreate(null);
                        //创建时间
                        crossSectionPO.setGmtModified(null);
                        crossSectionPOList.add(crossSectionPO);
                    }
                    planSectionVO.setCrossPlanPOList(crossPlanPOList);
                    planSectionVO.setCrossSectionPOList(crossSectionPOList);
                    list.add(planSectionVO);
                }
                return list;

            }
        }
        return null;
    }

    @Override
    public List<CrossSchedulesPO> crossSchedules(CrossSchedulesDTO crossSchedulesDTO) throws Exception {

        // 路口编号列表
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        // 信号机编号
        List<String> crossCodeList = crossInfoMapper.selectCrossCodesByIds(crossIdList);
        List<CrossSchedulesPO> results = new ArrayList<>(crossCodeList.size());
        if (!CollectionUtils.isEmpty(crossCodeList)) {
            for (String code : crossCodeList) {
                CrossInfoPO crossInfoBySignalCode = CrossInfoCache.getCrossInfoBySignalCode(code);
                String crossId = crossInfoBySignalCode.getId();
                LambdaQueryWrapper<VNtcipTimeBaseScheduleView> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(VNtcipTimeBaseScheduleView::getCIntsID, code);
                List<VNtcipTimeBaseScheduleView> vNtcipTimeBaseScheduleViews = vNtcipTimeBaseScheduleMapper.selectList(queryWrapper);
                if (!CollectionUtils.isEmpty(vNtcipTimeBaseScheduleViews)) {
                    for (VNtcipTimeBaseScheduleView vNtcipTimeBaseScheduleView : vNtcipTimeBaseScheduleViews) {
                        if (Objects.nonNull(vNtcipTimeBaseScheduleView)) {
                            CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
                            crossSchedulesPO.setCrossId(crossId);
                            crossSchedulesPO.setScheduleNo(vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                            crossSchedulesPO.setName("调度" + vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                            // 海信月份
                            Integer hisenseMonth = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleMonth();
                            // 星期
                            Integer hisenseWeek = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleDay();
                            // 日期 2147483647
                            Long hisenseDays = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleDate();
                            // 时段表号
                            Integer hisensePlanId = vNtcipTimeBaseScheduleView.getNTimeBaseScheduleDayPlan();
                            String monthsStr = ScheduleUtil.getSchedule(hisenseMonth.longValue(), 1);
                            String daysStr = ScheduleUtil.getSchedule(hisenseDays, 2);
                            String weekStr = ScheduleUtil.getSchedule(hisenseWeek.longValue(), 3);
                            if (StringUtils.isNotBlank(weekStr)) {
                                String[] weekSplit = weekStr.split(",");
                                for (String oneWeek : weekSplit) {
                                    CrossSchedulesPO crossSchedulesPOWeek = new CrossSchedulesPO();
                                    crossSchedulesPOWeek.setCrossId(crossId);
                                    crossSchedulesPOWeek.setScheduleNo(vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                    crossSchedulesPOWeek.setName("调度" + vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                    crossSchedulesPOWeek.setWeek(Integer.valueOf(oneWeek));
                                    crossSchedulesPOWeek.setMonths(monthsStr);
                                    crossSchedulesPOWeek.setDays(daysStr);
                                    crossSchedulesPOWeek.setPlanId(hisensePlanId);
                                    results.add(crossSchedulesPOWeek);
                                }
                            } else {
                                // 不执行周调度
                                CrossSchedulesPO crossSchedulesPOMonthDays = new CrossSchedulesPO();
                                crossSchedulesPOMonthDays.setCrossId(crossId);
                                crossSchedulesPOMonthDays.setScheduleNo(vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
                                crossSchedulesPOMonthDays.setName("调度" + vNtcipTimeBaseScheduleView.getNTimeBaseScheduleNumber());
//                                crossSchedulesPOMonthDays.setWeek(0);
//                                crossSchedulesPOMonthDays.setPlanId(hisensePlanId);
                                crossSchedulesPOMonthDays.setWeek(null);
                                crossSchedulesPOMonthDays.setPlanId(0);
                                crossSchedulesPOMonthDays.setMonths(monthsStr);
                                crossSchedulesPOMonthDays.setDays(daysStr);
                                results.add(crossSchedulesPOMonthDays);
                            }
                        }
                    }
                }
            }
        }
        return results;
    }


    @Override
    public CoordinationStatus crossPlan(String crossId) throws Exception {
        CoordinationStatus coordinationStatus = null;
        int schemeId = 0;
        //查询当前路口运行的方案
        LocalDate now = LocalDate.now();
        int week = now.getDayOfWeek().getValue();
//        if (week == 7) {
//            //表中周日用0表示
//            week = 0;
//        }
        CrossNowSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectByCrossIdAndWeek(crossId, week);
        if (ObjectUtil.isNotEmpty(baseCrossSchemePO)) {
            coordinationStatus = new CoordinationStatus();
            //从缓存中拿路口信息
            CrossInfoPO crossInfoPO = CrossInfoCache.getCrossInfoCache().get(crossId);
            if (ObjectUtil.isNotNull(crossInfoPO)) {
                coordinationStatus.setSpot(crossInfoPO.getCode());
                Integer poSchemeId = baseCrossSchemePO.getSchemeId();
                if (baseCrossSchemePO.getControlMode() == 3 || ObjectUtil.isEmpty(poSchemeId)) {
                    //控制模式为3或者方案号为空，判断为黃闪
                    coordinationStatus.setCoordPatternStatus(String.valueOf(255));
                } else {
                    //海信的方案号
                    int schemeNo = Integer.valueOf(baseCrossSchemePO.getSchemeId()) * 3 - 2;
                    coordinationStatus.setCoordPatternStatus(String.valueOf(schemeNo));
                }

                coordinationStatus.setCoordCycleStatus(String.valueOf(baseCrossSchemePO.getCycle()));
            }

        } else {
            coordinationStatus = crossPlanHisense(crossId);
        }
        return coordinationStatus;
    }

    /**
     * @return net.wanji.databus.po.CoordinationStatus
     * @Description 直接获取海信的私有协议
     * @Param [crossId]
     **/

    public CoordinationStatus crossPlanHisense(String crossId) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossId);
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_12);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.7. 协调状态 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String spot = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Spot");
            String CoordPatternStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordPatternStatus");
            String CoordCycleStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordCycleStatus");
            String CoordSyncStatus = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CoordSyncStatus");
            CoordinationStatus coordinationStatus = CoordinationStatus.builder()
                    .spot(crossId)
                    .coordPatternStatus(CoordPatternStatus)
                    .coordCycleStatus(CoordCycleStatus)
                    .coordSyncStatus(CoordSyncStatus)
                    .build();
            return coordinationStatus;

        } else {
            //失败
            log.error("请求 5.7. 协调状态 失败");
        }

        return null;
    }

    @Override
    public CrossSchemeRings crossSchemeRings(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossSchemeRingsDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_30);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        //获取当前运行的方案号
        Integer integer = planMap.get(crossSchemeRingsDTO.getCrossId());
        /*if (ObjectUtil.isEmpty(integer)){
            CoordinationStatus coordinationStatus = crossPlan(crossSchemeRingsDTO.getCrossId());
            if (ObjectUtil.isEmpty(coordinationStatus)) {
                //获取方案号失败
                log.error("请求方案环图前，获取当前运行方案号失败,crossId：{}", crossSchemeRingsDTO.getCrossId());
                return null;
            } else if ("255".equals(coordinationStatus.getCoordPatternStatus())) {
                //黃闪
                CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
                crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
                crossSchemeRings.setPattern(coordinationStatus.getCoordPatternStatus());
                return crossSchemeRings;
            }
            integer = Integer.valueOf(coordinationStatus.getCoordPatternStatus());
        }*/
        if (ObjectUtil.isEmpty(integer)){
                //获取方案号失败
                log.error("请求方案环图前，获取当前运行方案号失败,没有该路口的灯态信息，crossId：{}", crossSchemeRingsDTO.getCrossId());
                return null;
        } else if (255 == integer) {
            //黃闪
            CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
            crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
            crossSchemeRings.setPattern(String.valueOf(integer));
            return crossSchemeRings;
        }

        messageContent.addElement(HttpConstants.PATTERN).setText(String.valueOf(integer));

        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.15.根据方案号获取配时方案环图 失败,");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String patternRest = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Pattern");
            String cycleLen = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CycleLen");
            String ringCount = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("RingCount");
            String offset = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Offset");
            List<Element> cycleElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/CycleList");
            List<Element> phaseElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/PhaseList");
            List<Element> overLapPhaseElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/OverLapPhaseList");
            CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
            crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
            crossSchemeRings.setPattern(patternRest);
            crossSchemeRings.setRingCount(ringCount);
            crossSchemeRings.setCycleLen(cycleLen);
            crossSchemeRings.setOffset(offset);

            JSONObject jsonObject = new JSONObject();
            for (Element cycle : cycleElementList) {
                List<Node> elements = cycle.elements();
                for (Node node : elements) {
                    node.getName();
                    jsonObject.put(node.getName(), node.getText());
                }
            }
            List<CrossSchemeRings.Phase> phaseList = new ArrayList<>();
            for (Element el : phaseElementList) {
                CrossSchemeRings.Phase phase = new CrossSchemeRings.Phase();
                phase.setRedTime(el.valueOf("RedTime"));
                phase.setYellowTime(el.valueOf("YellowTime"));
                phase.setGreenTime(el.valueOf("GreenTime"));
                phase.setDirection(el.valueOf("Direction"));
                phase.setMinGTime(el.valueOf("MinGTime"));
                phase.setMaxGTime(el.valueOf("MaxGTime"));
                phase.setChannelDim(el.valueOf("ChannelDim"));
                phase.setConcurrent(el.valueOf("Concurrent"));
                phaseList.add(phase);
            }
            List<CrossSchemeRings.OverLapPhase> overLapPhaseList = new ArrayList<>();
            for (Element el : overLapPhaseElementList) {
                CrossSchemeRings.OverLapPhase overLapPhase = new CrossSchemeRings.OverLapPhase();
                overLapPhase.setDirection(el.valueOf("Direction"));
                overLapPhase.setIncludedPhases(el.valueOf("IncludedPhases"));
                overLapPhase.setModifierPhases(el.valueOf("ModifierPhases"));
                overLapPhaseList.add(overLapPhase);
            }
            crossSchemeRings.setCycleList(jsonObject);
            crossSchemeRings.setPhaseList(phaseList);
            crossSchemeRings.setOverLapPhaseList(overLapPhaseList);
            return crossSchemeRings;

        } else {
            //失败
            log.error("请求 5.15. 方案环图请求 失败");
        }
        return null;
    }

    /**
     * @return net.wanji.databus.po.CrossSchemeRings
     * @Description 获取环图请求原有的逻辑
     * @Param [crossSchemeRingsDTO]
     **/

    public CrossSchemeRings crossSchemeRingsHisen(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        CrossInfoPOExt crossInfoPOExt = crossInfoMapper.selectById(crossSchemeRingsDTO.getCrossId());
        //组装document
        Document document = OperationBaseDom.generateBaseDoc();
        Element element = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION);
        element.addElement(HttpConstants.MESSAGETYPE).setText(HttpConstants.MESSAGETYPE_30);
        Element messageContent = (Element) document.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT);
        messageContent.addElement(HttpConstants.SPOT).setText(crossInfoPOExt.getCode());
        messageContent.addElement(HttpConstants.PATTERN).setText(crossSchemeRingsDTO.getPattern());
        //给海信发送http请求
        String post = OkHttpClientUtil.xmlPost(hisenseUrl, document.asXML());
        if (StringUtils.isBlank(post)) {
            log.error("请求 5.15.根据方案号获取配时方案环图 失败");
            return null;
        }
        Document postResult = DocumentHelper.parseText(post);
        String result = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_RESULT).getText();
        if (HttpConstants.RESULT_1.equals(result)) {
            //成功
            String patternRest = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Pattern");
            String cycleLen = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("CycleLen");
            String ringCount = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("RingCount");
            String offset = postResult.selectSingleNode(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT).valueOf("Offset");
            List<Element> cycleElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/CycleList");
            List<Element> phaseElementList = postResult.selectNodes(HttpConstants.SYSTEMSCRIPTION_MESSAGECONTENT + "/PhaseList");
            CrossSchemeRings crossSchemeRings = new CrossSchemeRings();
            crossSchemeRings.setSpot(crossSchemeRingsDTO.getCrossId());
            crossSchemeRings.setPattern(patternRest);
            crossSchemeRings.setRingCount(ringCount);
            crossSchemeRings.setCycleLen(cycleLen);
            crossSchemeRings.setOffset(offset);

            JSONObject jsonObject = new JSONObject();
            for (Element cycle : cycleElementList) {
                List<Node> elements = cycle.elements();
                for (Node node : elements) {
                    node.getName();
                    jsonObject.put(node.getName(), node.getText());
                }
            }
            List<CrossSchemeRings.Phase> phaseList = new ArrayList<>();
            for (Element el : phaseElementList) {
                CrossSchemeRings.Phase phase = new CrossSchemeRings.Phase();
                phase.setRedTime(el.valueOf("RedTime"));
                phase.setYellowTime(el.valueOf("YellowTime"));
                phase.setGreenTime(el.valueOf("GreenTime"));
                phase.setDirection(el.valueOf("Direction"));
                phase.setMinGTime(el.valueOf("MinGTime"));
                phase.setMaxGTime(el.valueOf("MaxGTime"));
                phase.setChannelDim(el.valueOf("ChannelDim"));
                phase.setConcurrent(el.valueOf("Concurrent"));
                phaseList.add(phase);
            }
            crossSchemeRings.setCycleList(jsonObject);
            crossSchemeRings.setPhaseList(phaseList);
            return crossSchemeRings;

        } else {
            //失败
            log.error("请求 5.15. 方案环图请求 失败");
        }
        return null;
    }

    @Override
    public List<CrossSchemeStageOptLogPO> schemeOptLog(CrossSchemeRingsDTO crossSchemeRingsDTO) throws Exception {
        LambdaQueryWrapper<CrossSchemeStageOptLogPO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(CrossSchemeStageOptLogPO::getCrossId, crossSchemeRingsDTO.getCrossId())
                .eq(CrossSchemeStageOptLogPO::getSchemeId, crossSchemeRingsDTO.getPattern())
                .inSql(CrossSchemeStageOptLogPO::getBatchTime, "select max(batch_time) from t_cross_scheme_opt_log where cross_id = '" + crossSchemeRingsDTO.getCrossId() + "' and scheme_id = " + crossSchemeRingsDTO.getPattern());
        List<CrossSchemeStageOptLogPO> crossSchemeStageOptLogPOS = crossSchemeStageOptLogMapper.selectList(queryWrapper);
        return crossSchemeStageOptLogPOS;
    }

    @Override
    public BaseCrossSchemePO selectSchemeByParams(QueryByCrossIdAndTimeDTO queryByCrossIdAndTimeDTO) throws Exception {
        Date datetime = queryByCrossIdAndTimeDTO.getDatetime();
        //java.util.date转LocalDate
        LocalDate localDate = datetime.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
        int week = localDate.getDayOfWeek().getValue();
        if (week == 7) {
            //表中周日用0表示
            week = 0;
        }
        // 创建一个 SimpleDateFormat 对象，并指定日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 将 Date 对象格式化为字符串
        String formattedDate = sdf.format(datetime);
        BaseCrossSchemePO baseCrossSchemePO = baseCrossSchemeMapper.selectSchemeByParams(queryByCrossIdAndTimeDTO.getCrossId(), formattedDate, week);
        if (ObjectUtil.isNull(baseCrossSchemePO)) {
            //获取当前方案号
            CoordinationStatus coordinationStatus = crossPlan(queryByCrossIdAndTimeDTO.getCrossId());
            if (ObjectUtil.isNotNull(coordinationStatus)) {
                baseCrossSchemePO = new BaseCrossSchemePO();
                String coordPatternStatus = coordinationStatus.getCoordPatternStatus();
                int schemaId = (Integer.valueOf(coordPatternStatus) + 2) / 3;
                baseCrossSchemePO.setSchemeNo(String.valueOf(schemaId));
                baseCrossSchemePO.setCrossId(queryByCrossIdAndTimeDTO.getCrossId());
                baseCrossSchemePO.setCycle(Integer.valueOf(coordinationStatus.getCoordCycleStatus()));
            }
        }
        return baseCrossSchemePO;
    }
}
